#' # This script aims to verify the quality and preprocess the microarray data on PBMC. Use the spin() function to create the report (i.e. spin(3c.PreProcessMicroarray.R))
#+ setuppeeee, include=FALSE
#opts_chunk$set(echo=FALSE, dev = c('png', 'pdf')) 
#opts_knit$set(root.dir="~/Desktop/Work_temp/Deconvolution/Microarrays", progress = FALSE)

#' reading two batches of microarray. One processed on the 7-Sep-2016 and one processed on the 23-Dec-2016. The expiration date for the microarray chips was the 30th of september. There is the possibility that the second batch might have some problems.

 setwd("~/Desktop/Work_temp/Deconvolution/Microarrays")
#+ include=FALSE
library(lumi)
library(limma)
library(sva)
batchSept <- lumiR("E0162-16 AL Gianni Raw.txt", annotationColumn=c('ENTREZ_GENE_ID', 'SYMBOL'))
batchDec <- lumiR("E0162-16 AL Gianni B2 Raw.txt", annotationColumn=c('ENTREZ_GENE_ID', 'SYMBOL'))
#' number of probes for batch analyzed in september
nrow(batchSept)
#' number of probes for batch analyzed in december
nrow(batchDec)

### prepare data for GEO submission
batchC0 <- combine(batchDec, batchSept[,9])
namesC0 <- substring(sampleNames(phenoData(batchC0)), 1, 4)
samples <- c("9JD4","925L","G4YW","DZQV","453W","684C","CZJE","FY2H","CYFZ","FLWA","4DUY","36TS","CR3L")
batchC0ord <- batchC0[,match(samples, namesC0)]
#sampleNames(phenoData(batchC0ord)) 
exprsV <- exprs(batchC0ord)
detectV <- get('detection', assayData(batchC0ord))  # this is to extract the detection p-value
colnames(detectV) <- paste0(samples, "_Detection_Pval")
rows.combined <- nrow(exprsV) 
cols.combined <- ncol(exprsV) + ncol(detectV)
matrix.combined <- matrix(NA, nrow=rows.combined, ncol=cols.combined)
matrix.combined[, seq(1, cols.combined, 2)] <- exprsV
matrix.combined[, seq(2, cols.combined, 2)] <- detectV
cols <- rep("pippo", ncol(matrix.combined))
cols[seq(1, cols.combined, 2)] <- samples
cols[seq(2, cols.combined, 2)] <- paste0(samples, "_Detection_Pval")
colnames(matrix.combined) <- cols
rownames(matrix.combined) <- rownames(exprsV)
write.table(matrix.combined, file= "GEO_raw_data.txt", sep="\t", col.names=NA)

batchC0ordT <- lumiN(batchC0ord, method="quantile")
exprsVT <- exprs(batchC0ordT)
matrix.combined2 <- matrix.combined
matrix.combined2[, seq(1, cols.combined, 2)] <- exprsVT
write.table(matrix.combined2, file= "GEO_quantile_data.txt", sep="\t", col.names=NA)

#' 
#'Only the probes with a detection p-value < 0.1 in at least 30 % of samples were kept.
detectionRate <- 0.3
detection.p <- 0.1
detect.countSept <- detectionCall(batchSept,Th=detection.p,type="probe")
detect.rateSept <- detect.countSept/ncol(exprs(batchSept))
#' number of probes left for the September batch:
sum(detect.rateSept>=detectionRate)
detect.countDec <- detectionCall(batchDec,Th=detection.p,type="probe")
detect.rateDec <- detect.countDec/ncol(exprs(batchDec))
#' number of probes left for the December batch:
sum(detect.rateDec>=detectionRate)

# keep probes that are expressed in at least one sample
batchSept2 <- batchSept[detect.countSept>=1, ] 
batchDec2 <- batchDec[detect.countDec>=1, ] 

#' 
#' Let's how many unique ENTREZ genes I have.
#' For the September batch:
length(unique(fData(batchSept2)$ENTREZ_GENE_ID))
#' For the December batch:
length(unique(fData(batchDec2)$ENTREZ_GENE_ID))
#' 

#' Now I combine the two dataset using only the probes in common and I add the phenotype data. I keep all the rows available (meaning I don't apply the filtering with detection p.value)
sub <- intersect(rownames(batchSept2), rownames(batchDec2))
batchC <- combine(batchSept2[sub,],batchDec2[sub,]) # combine  the two datasets
pheno <- read.delim("Pheno_microarray.txt",row.names = 1)
pheno$id  <- 1:nrow(pheno)
phenomerge <- merge(pData(batchC), pheno, by="row.names")
phenomerge <- phenomerge[order(phenomerge$id),]
phenomerge$sampleID <- NULL
rownames(phenomerge) <- phenomerge$Row.names
pData(batchC) <- phenomerge
symb <- featureData(batchC)@data

#+ fig.width = 15, fig.height = 5
par(mfrow=c(1,2))
plot(batchC, what='boxplot') 
plotSampleRelation(batchC)
#' With this plot is comforting to see the largest variation is still due to the amplification protocol used rather than the date of processing.
#' 
#' The plots were produced on raw data, let's see the results after normalization and log transformation
batchCT <- lumiN(batchC, method="quantile")
batchCTN <- lumiT(batchCT, method="log2")
#+ fig.width = 15, fig.height = 5
par(mfrow=c(1,2))
plot(batchCTN, what='boxplot') 
plotSampleRelation(batchCTN)
#' the hiearchical clustering shows even better results now as all the samples processed with TRIzol and epi 2 cycle cluster together
#' 
#' let's look at pairs between some samples of the two batches processed with Trizol and Epi 2 cycle
pairs(batchCTN[,c(7,8,9,13,14,15)], labels= pData(batchCTN)$Date_acquisition[c(7,8,9,13,14,15)])
#' They look fine
  
#' Let's keep only the samples processed with TRIzol and Epi 2 cycle now. I take the raw data, subset them and then normalize and log transform. 
Csub <- batchC[, rownames(subset(pData(batchC), Lysis_buffer=="Trizol" & Amplification_protocol=="Epi 2 cycle"))]
colnames(Csub) <- pData(Csub)$sample
rownames(pData(Csub)) <- pData(Csub)$sample
pData(Csub)$Row.names <- NULL
pData(Csub)$sample <- NULL
pData(Csub)$id <- NULL
CTsub <- lumiN(Csub, method="quantile")
CTNsub <- lumiT(CTsub, method="log2")
#+ fig.width = 15, fig.height = 5
par(mfrow=c(1,2))
plot(CTNsub, what='boxplot') 
plotSampleRelation(CTNsub, labels= pData(CTNsub)$Date_acquisition)
#' now you can notice the batch effect due to date of processing
#' 

#'## Remove batch effect with ComBat

#' ComBat can remove the batch effect from known batches. 
batch = pData(CTNsub)$Date_acquisition
combat_edata = ComBat(dat=exprs(CTNsub), batch=batch, par.prior=T, prior.plots=F)
exprs(CTNsub) <- combat_edata
plotSampleRelation(CTNsub, labels = pData(CTNsub)$Date_acquisition)
#' The data are now corrected
#'
  
#' I save the lumi object normalized, log transformed and corrected for batch effect for future analysis as Comb_TN.RData
save(CTNsub, file="Comb_TN.RData")
# save(Csub, file="Comb.RData") # no normalization and no transformation





# #'Only the probes with a detection p-value < 0.1 in at least 30 % of samples were kept.
# detectionRate <- 0.3
# detection.p <- 0.1
# detect.countSept <- detectionCall(batchSept,Th=detection.p,type="probe")
# detect.rateSept <- detect.countSept/ncol(exprs(batchSept))
# #' number of probes left for the September batch:
# sum(detect.rateSept>=detectionRate)
# detect.countDec <- detectionCall(batchDec,Th=detection.p,type="probe")
# detect.rateDec <- detect.countDec/ncol(exprs(batchDec))
# #' number of probes left for the December batch:
# sum(detect.rateDec>=detectionRate)
# 
# # if you want to remove the probes with missing signal
# batchSept <- batchSept[detect.rateSept>=detectionRate, ] 
# batchDec <- batchDec[detect.rateDec>=detectionRate, ] 
